<?php
namespace ElementorHoldy;

class HoldyLoadElementor {

	private static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function widget_styles() {
		wp_register_style( 'spectrum', plugins_url( '/css/spectrum.min.css', __FILE__ ), false, '2.0.8' );
        wp_register_style( 'holdy-generator', plugins_url( '/css/generator.css', __FILE__ ), false, HOLDY_VERSION );
		wp_register_style( 'holdy-image', plugins_url( '/css/image.css', __FILE__ ), false, HOLDY_VERSION );
	}

	public function widget_scripts() {
		wp_register_script( 'spectrum', plugins_url( '/js/spectrum.min.js', __FILE__ ), [ 'jquery' ], false, true );
        wp_register_script( 'holdy-generator', plugins_url( '/js/generator.js', __FILE__ ), [ 'jquery' ], false, true );
	}

	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/image.php' );
		require_once( __DIR__ . '/widgets/generator.php' );
	}

	public function register_widgets() {
		$this->include_widgets_files();
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\Holdy_Image() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\Holdy_Generator() );
	}

	public function widget_category($elements_manager) {
		$elements_manager->add_category(
			'holdy-widgets',
			[
				'title' => esc_html__( 'Holdy', 'holdy' ),
				'icon' => 'fa fa-plug',
			]
		);
	}

	public function __construct() {
		add_action( 'elementor/frontend/after_enqueue_styles', [ $this, 'widget_styles' ] );
		add_action( 'elementor/frontend/after_register_scripts', [ $this, 'widget_scripts' ] );
		add_action( 'elementor/elements/categories_registered', [ $this, 'widget_category' ] );
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'register_widgets' ] );
	}
}

HoldyLoadElementor::instance();