<?php
defined( 'ABSPATH' ) || exit;

use GDText\Box;
use GDText\Color;

class HoldyPlaceholder {
    /**
	 * The single instance of the class
	 */
	protected static $_instance = null;

    /**
	 * Main Instance
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /**
	 * Construct
	 */
    public function __construct() {
        add_action('init', array($this, 'init'), 1);
        add_action('init', array($this, 'placeholder'), 2);
        add_action('plugins_loaded', array($this, 'elementor_init'));
        add_action('admin_menu', array($this, 'register_page'));
        add_action('wp_ajax_holdySave', array($this, 'save'));
    }

    /**
	 * Init
	 */
    public function init() {
        load_plugin_textdomain( 'holdy', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
    }

    /**
	 * Register Admin Page
	 */
    public function register_page(){
        add_menu_page( 
            esc_html__( 'Placeholder', 'holdy' ),
            esc_html__( 'Placeholder', 'holdy' ),
            'manage_options',
            'holdy_placeholder',
            array($this, 'page_output'),
            'dashicons-align-center',
            10
        ); 
    }

    /**
	 * Page Output
	 */
    public function page_output(){
        $slug = HoldySettings::get_option('holdy_slug', 'placeholder');
        $size = HoldySettings::get_option('holdy_image_size', '800x600');
        ?>
        <div id="holdy-admin-generator">
        <h2><?php echo esc_html__( 'Placeholder Image Generator', 'holdy' ); ?></h2>
        <div class="holdy-list-wrapper holdy-list-wrapper-row" data-url="<?php echo esc_url(get_site_url() . '/' . $slug . '/'); ?>" data-size="<?php echo esc_attr($size); ?>">
			<div class="holdy-list-container">
				<div class="holdy-form-wrapper">
				<div class="holdy-form-field-wrap">
					<div class="holdy-form-field">
						<label class="holdy-input-label"><?php echo esc_html__( 'Width', 'holdy' ); ?></label>
						<input class="holdy-input-text holdy-numeric" type="number" name="holdy_width" value="" autocomplete="off" max="4000">
					</div>
					<div class="holdy-form-field">
						<label class="holdy-input-label"><?php echo esc_html__( 'Height', 'holdy' ); ?></label>
						<input class="holdy-input-text holdy-numeric" type="number" name="holdy_height" value="" autocomplete="off" max="4000">
					</div>
				</div>
				<div class="holdy-form-field-wrap">
					<div class="holdy-form-field">
						<label class="holdy-input-label"><?php echo esc_html__( 'Background', 'holdy' ); ?></label>
						<input class="holdy-colorpicker spectrum with-add-on" type="text" name="holdy_background" value="" autocomplete="off">
					</div>
					<div class="holdy-form-field">
						<label class="holdy-input-label"><?php echo esc_html__( 'Format', 'holdy' ); ?></label>
						<select class="holdy-select" name="holdy_format" autocomplete="off">
                        <?php 
						$formats = $this->get_supported_formats(); 
						foreach($formats as $key => $item) {
							$selected = '';
							if ($key == $format) {
								$selected = 'selected';
							}
							echo '<option value="' . esc_attr($key) . '" ' . esc_attr($selected) . '>' . esc_html($item) . '</option>';
						}
						?>
                        </select>
					</div>
				</div>
				<div class="holdy-form-field-wrap">
					<div class="holdy-form-field">
						<label class="holdy-input-label"><?php echo esc_html__( 'Font Family', 'holdy' ); ?></label>
						<select class="holdy-select" name="holdy_font_family" autocomplete="off">
                        <?php 
						$font_family_array = $this->get_fonts(); 
						foreach($font_family_array as $key => $item) {
							$selected = '';
							if ($key == $parameters['font']) {
								$selected = 'selected';
							}
							echo '<option value="' . esc_attr($key) . '" ' . esc_attr($selected) . '>' . esc_html($item) . '</option>';
						}
						?>
                        </select>
					</div>
					<div class="holdy-form-field">
                        <label class="holdy-input-label"><?php echo esc_html__( 'Font Size', 'holdy' ); ?></label>
                        <input class="holdy-input-text holdy-numeric" type="number" name="holdy_font_size" value="" autocomplete="off" max="1000">
					</div>
				</div>
				<div class="holdy-form-field-wrap">
					<div class="holdy-form-field">
                        <label class="holdy-input-label"><?php echo esc_html__( 'Text Color', 'holdy' ); ?></label>
                        <input class="holdy-colorpicker spectrum with-add-on" type="text" name="holdy_font_color" value="" autocomplete="off">
					</div>
					<div class="holdy-form-field">
                        <label class="holdy-input-label"><?php echo esc_html__( 'Text Align', 'holdy' ); ?></label>
                        <select class="holdy-select" name="holdy_align" autocomplete="off">
                        <?php 
						$alignment_array = $this->get_alignments(); 
						foreach($alignment_array as $key => $item) {
							$selected = '';
							if ($key == $parameters['align']) {
								$selected = 'selected';
							}
							echo '<option value="' . esc_attr($key) . '" ' . esc_attr($selected) . '>' . esc_html($item) . '</option>';
						}
						?>
                        </select>
					</div>
				</div>
				<div class="holdy-form-field-wrap">
					<div class="holdy-form-field">
                        <label class="holdy-input-label"><?php echo esc_html__( 'Stroke Color', 'holdy' ); ?></label>
                        <input class="holdy-colorpicker spectrum with-add-on" type="text" name="holdy_stroke_color" value="" autocomplete="off">
					</div>
					<div class="holdy-form-field">
                        <label class="holdy-input-label"><?php echo esc_html__( 'Stroke Size', 'holdy' ); ?></label>
                        <input class="holdy-input-text holdy-numeric" type="number" name="holdy_stroke_size" value="" autocomplete="off" max="4000">
					</div>
				</div>
				<div class="holdy-form-field ">
                        <label class="holdy-input-label"><?php echo esc_html__( 'Text', 'holdy' ); ?></label>
                        <textarea class="holdy-textarea" name="holdy_text" autocomplete="off"></textarea>
				</div>
				<div class="holdy-form-field-wrap copy-field">
					<div class="holdy-form-field">
						<input class="holdy-input-text" type="text" name="holdy_placeholder_url" value="" autocomplete="off" readonly>
					</div>
					<div class="holdy-form-field">
						<button type="button" class="holdy-btn" name="holdy_copy_url" data-text="<?php echo esc_attr__( 'Copy URL', 'holdy' ); ?>" data-copied="<?php echo esc_attr__( 'Copied!', 'holdy' ); ?>"><?php echo esc_html__( 'Copy URL', 'holdy' ); ?></button>
					</div>
				</div>
                <div class="holdy-form-field-wrap">
					<div class="holdy-form-field">
						<button type="button" class="holdy-btn" name="holdy_img_view"><?php echo esc_html__( 'View', 'holdy' ); ?></button>
					</div>
                    <div class="holdy-form-field">
						<button type="button" class="holdy-btn" name="holdy_img_download"><?php echo esc_html__( 'Download', 'holdy' ); ?></button>
					</div>
                    <div class="holdy-form-field">
						<button type="button" class="holdy-btn primary" name="holdy_img_save" data-text="<?php echo esc_attr__( 'Save', 'holdy' ); ?>" data-saving="<?php echo esc_attr__( 'Saving...', 'holdy' ); ?>" data-saved="<?php echo esc_attr__( 'Saved!', 'holdy' ); ?>" data-ajax="<?php echo esc_url(admin_url( 'admin-ajax.php' )); ?>" data-nonce="<?php echo wp_create_nonce('holdy-nonce'); ?>"><?php echo esc_html__( 'Save', 'holdy' ); ?></button>
					</div>
				</div>
				</div>
			</div>
		</div>
        </div>
    <?php }

    /**
	 * Random hex color generator
	 */
    public function randomHex() {
        return '#' . str_pad(dechex(mt_rand(0, 0xFFFFFF)), 6, '0', STR_PAD_LEFT);
    }

    /**
     * Uses the luminosity algorithm to determine the hex contrast color
     * for a given background color.
     *
     * @return string
     */
    public function contrastColor($hex) {
        if ($hex == null) {
            $hex = "#000000";
        }

        $hex = str_replace('#', '', $hex);

        $R1 = hexdec(substr($hex, 0, 2));
        $G1 = hexdec(substr($hex, 2, 2));
        $B1 = hexdec(substr($hex, 4, 2));

        $blackColor = "000000";
        $R2BlackColor = hexdec(substr($blackColor, 0, 2));
        $G2BlackColor = hexdec(substr($blackColor, 2, 2));
        $B2BlackColor = hexdec(substr($blackColor, 4, 2));

        $L1 = 0.2126 * pow($R1 / 255, 2.2) +
            0.7152 * pow($G1 / 255, 2.2) +
            0.0722 * pow($B1 / 255, 2.2);

        $L2 = 0.2126 * pow($R2BlackColor / 255, 2.2) +
            0.7152 * pow($G2BlackColor / 255, 2.2) +
            0.0722 * pow($B2BlackColor / 255, 2.2);

        if ($L1 > $L2) {
            $contrastRatio = (int)(($L1 + 0.05) / ($L2 + 0.05));
        } else {
            $contrastRatio = (int)(($L2 + 0.05) / ($L1 + 0.05));
        }

        if ($contrastRatio > 5) {
            return '#000';
        } else {
            return '#FFF';
        }
    }

    /**
     * Allocates a specific hex color for gd.
     * You must have allocated the resource beforehand.
     * Returns an int representing the desired color.
     *
     * @param $resource
     * @param $hex
     * @return false|int
     */
    public function allocate($resource, $hex) {
        $rgbArray = $this->toRgbArray($hex);
        return imagecolorallocate($resource, $rgbArray['r'], $rgbArray['g'], $rgbArray['b']);
    }

    /**
     * Converts a hex color string to an array (r: int, g: int, b: int).
     * This array can then be used if you need rgb values.
     *
     * Returns false if the hex cannot be converted (e.g. invalid length).
     *
     * @param $hex
     * @return array|bool
     */
    public function toRgbArray($hex) {
        $hex = preg_replace("/[^abcdef0-9]/i", "", $hex);
        if (strlen($hex) == 6) {
            list($r, $g, $b) = str_split($hex, 2);
            return [
                "r" => hexdec($r),
                "g" => hexdec($g),
                "b" => hexdec($b),
            ];
        } elseif (strlen($hex) == 3) {
            list($r, $g, $b) = array($hex[0] . $hex[0], $hex[1] . $hex[1], $hex[2] . $hex[2]);
            return [
                "r" => hexdec($r),
                "g" => hexdec($g),
                "b" => hexdec($b),
            ];
        }
        return false;
    }

    /**
     * Generates an image; directly renders a placeholder image.
     *
     * @return bool
     */
    public function generate($ext = "png", $text = "", $size = null, $bgHex = null, $textHex = null, $fontFamily = "lato", $fontSize = null, $align = "center-center", $strokeHex = null, $strokeSize = null){
        $targetSize = empty($size) ? HoldySettings::get_option('holdy_image_size', '800x600') : $size;
        $allowedSize = HoldySettings::get_option('holdy_size_limit', 4000);
        $align = explode('-', $align);
        $h_align = $align[0];
        $v_align = $align[1];
        $dimensions = explode('x', $targetSize);
        if (!isset($dimensions[1])) {
            if ($dimensions[0] > $allowedSize) {
                wp_die(esc_html__('Maximum allowed image size is', 'holdy') . ' ' . $allowedSize . 'x' . $allowedSize . 'px.');
            }
            $imageResource = imagecreate($dimensions[0], $dimensions[0]);
        } else {
            if ($dimensions[0] > $allowedSize || $dimensions[1] > $allowedSize) {
                wp_die(esc_html__('Maximum allowed image size is', 'holdy') . ' ' . $allowedSize . 'x' . $allowedSize . 'px.');
            }
            $imageResource = imagecreate($dimensions[0], $dimensions[1]);
        }

        $randomColor = $this->randomHex();
        $bgColor = ! empty($bgHex) ? $bgHex : $randomColor;
        $fgColor = ! empty($textHex) ? $textHex : $this->contrastColor($bgColor);

        if ($text == "") {
            $text = $targetSize;
        }

        $this->allocate($imageResource, $bgColor);

        $arrayFgColor = $this->toRgbArray($fgColor);
        $fontPath = $this->get_font_by_id($fontFamily);

        if ($fontPath !== null && file_exists($fontPath)) {
            $font = $fontPath;

            if ($fontSize == null) {
                if (imagesx($imageResource) >= imagesy($imageResource)) {
                    $fontSize = imagesy($imageResource) / 8;
                } else {
                    $fontSize = imagesx($imageResource) / 8;
                }
            }

            $box = new Box($imageResource);
            $box->setFontFace($font);
            $box->setFontColor(new Color($arrayFgColor["r"], $arrayFgColor["g"], $arrayFgColor["b"]));
            $box->setFontSize($fontSize);
            $box->setBox(0, 0, imagesx($imageResource), imagesy($imageResource));
            $box->setTextAlign($h_align, $v_align);
            $box->setLineHeight(1);
            if ($strokeHex !== null && $strokeHex != '' && $strokeSize !== 0 && $strokeSize !== null && $strokeSize != '') {
                $arraystrokeHex = $this->toRgbArray($strokeHex);
                $box->setStrokeColor(new Color($arraystrokeHex["r"], $arraystrokeHex["g"], $arraystrokeHex["b"]));
                $box->setStrokeSize((int) $strokeSize);
            }
            // $box->setBackgroundColor(new Color(255, 86, 77));
            $box->draw($text);
           
        } else {
            $allocatedFgColor = $this->allocate($imageResource, $fgColor);
            $fontSize = 5;
            $fontWidth  = imagefontwidth($fontSize);
            $fontHeight = imagefontwidth($fontSize);
            $length = strlen($text);
            $textWidth = $length * $fontWidth;
            $x = (imagesx($imageResource) - $textWidth) / 2;
            $y = (imagesy($imageResource) - $fontHeight) / 2;
            imagestring(
                $imageResource,
                $fontSize,
                (int) $x,
                (int) $y,
                $text,
                $allocatedFgColor
            );
        }

        // Render image
        if ($ext == 'jpg' || $ext == 'jpeg') {
            header('Content-type: image/jpeg');
            echo imagejpeg($imageResource, null);
        } else if ($ext == 'gif') {
            header('Content-type: image/gif');
            echo imagegif($imageResource, null);
        } else if ($ext == 'bmp') {
            header('Content-type: image/bmp');
            echo imagebmp($imageResource, null);
        } else if ($ext == 'webp') {
            header('Content-type: image/webp');
            imagepalettetotruecolor($imageResource);
			imagealphablending($imageResource, true);
			imagesavealpha($imageResource, true);
            echo imagewebp($imageResource, null);
        } else {
            header('Content-type: image/png');
            echo imagepng($imageResource, null);
        }
        exit();
    }

    /**
     * Catches our query variable. If it’s there, we’ll stop the
     * rest of WordPress from loading and return the placeholder
     */
    public function placeholder() {
        $slug =  HoldySettings::get_option('holdy_slug', 'placeholder');
        $requestURI = $_SERVER['REQUEST_URI'];
        if($requestURI && !empty($requestURI) && !is_admin() && !wp_doing_ajax()) {
            $pieces = explode("/", $requestURI );
            if (in_array($slug, $pieces)) {
                do_action('holdy_before');
                $supported_image = array('gif','jpg','jpeg','png','bmp','webp');   
                $src_file_name = end($pieces);
                $ext = strtolower(pathinfo($src_file_name, PATHINFO_EXTENSION));
                $size = pathinfo($src_file_name, PATHINFO_FILENAME);
                $queries = array();
                parse_str($_SERVER['QUERY_STRING'], $queries);
                $text = HoldySettings::get_option('holdy_text', '');
                $bgColor = HoldySettings::get_option('holdy_background_color', null);
                $textColor = HoldySettings::get_option('holdy_font_color', null);
                $fontSize = null;
                $fontFamily = HoldySettings::get_option('holdy_font_family', 'lato');
                $align = HoldySettings::get_option('holdy_align', 'center-center');
                $strokeColor = HoldySettings::get_option('holdy_stroke_color', null);
                $strokeSize = HoldySettings::get_option('holdy_stroke_size', null);
                if (!empty($queries)) {
                    if (isset($queries['text']) && !empty($queries['text'])) {
                        $text = str_replace("$5Cn", "\n", $queries['text']);
                    }
                    if (isset($queries['background']) && !empty($queries['background'])) {
                        $bgColor = '#' . $queries['background'];
                    }
                    if (isset($queries['foreground']) && !empty($queries['foreground'])) {
                        $textColor = '#' . $queries['foreground'];
                    }
                    if (isset($queries['font']) && !empty($queries['font'])) {
                        $fontFamily = $queries['font'];
                    }
                    if (isset($queries['fontsize']) && !empty($queries['fontsize'])) {
                        $fontSize = $queries['fontsize'];
                    }
                    if (isset($queries['align']) && !empty($queries['align'])) {
                        $align = $queries['align'];
                    }
                    if (isset($queries['strokecolor']) && !empty($queries['strokecolor'])) {
                        $strokeColor = $queries['strokecolor'];
                        if (isset($queries['strokesize']) && !empty($queries['strokesize'])) {
                            $strokeSize = $queries['strokesize'];
                        }
                    }
                }
                if (in_array(strtok($ext, '?'), $supported_image)) {
                    $this->generate(strtok($ext, '?'), $text, $size, $bgColor, $textColor, $fontFamily, $fontSize, $align, $strokeColor, $strokeSize);
                } else {
                    $this->generate('png', $text, $size, $bgColor, $textColor, $fontFamily, $fontSize, $align, $strokeColor, $strokeSize);
                };
                do_action('holdy_after');
                exit();
            } else {
                return;
            }
        } else {
            return;
        }
    }

    // Save placeholder image
    public function save() {
        if ( ! wp_verify_nonce( $_POST['nonce'], 'holdy-nonce' ) ) {
            wp_die(esc_html__('Security Error!', 'holdy'));
        }
        $upload_dir  = wp_upload_dir();
        $upload_path = str_replace( '/', DIRECTORY_SEPARATOR, $upload_dir['path'] ) . DIRECTORY_SEPARATOR;

        // Save the image in the uploads directory.
        $img = file_get_contents( $_POST['url']);
        $upload_file = file_put_contents( $upload_path . $_POST['name'], $img );

        // Save attachement
        $wp_filetype = wp_check_filetype($_POST['name'], null );
        $attachment = array(
            'post_mime_type' => $wp_filetype['type'],
            'post_title'     => esc_html($_POST['name']),
            'post_content'   => '',
            'post_status'    => 'inherit',
            'guid'           => $_POST['url']
        );
        $attachment_id = wp_insert_attachment( $attachment, $upload_dir['path'] . '/' . $_POST['name'] );
        wp_update_attachment_metadata(
            $attachment_id,
            wp_generate_attachment_metadata( $attachment_id, $upload_dir['path'] . '/' . $_POST['name'] )
        );
        exit();
    }

    // Get list of fonts
    static function get_fonts() {
        $fonts = apply_filters('holdyFonts',array(
            '' => esc_html__( 'Select Font', 'holdy' ),
            'lato' => esc_html__( 'Lato', 'holdy' ),
            'lora' => esc_html__( 'Lora', 'holdy' ),
            'montserrat' => esc_html__( 'Montserrat', 'holdy' ),
            'opensans' => esc_html__( 'Open Sans', 'holdy' ),
            'oswald' => esc_html__( 'Oswald', 'holdy' ),
            'playfair' => esc_html__( 'Playfair Display', 'holdy' ),
            'ptsans' => esc_html__( 'PT Sans', 'holdy' ),
            'raleway' => esc_html__( 'Raleway', 'holdy' ),
            'roboto' => esc_html__( 'Roboto', 'holdy' ),
            'code' => esc_html__( 'Source Code Pro', 'holdy' ),
        ));
        return $fonts;
    }

    // Get text alignments
    static function get_alignments() {
        $alignments = apply_filters('holdyAlign',array(
            'center-center' => esc_html__( 'Center', 'holdy' ),
            'left-center' => esc_html__( 'Left Middle', 'holdy' ),
            'right-center' => esc_html__( 'Right Middle', 'holdy' ),
            'left-top' => esc_html__( 'Left Top', 'holdy' ),
            'center-top' => esc_html__( 'Center Top', 'holdy' ),
            'right-top' => esc_html__( 'Right Top', 'holdy' ),
            'left-bottom' => esc_html__( 'Left Bottom', 'holdy' ),
            'center-bottom' => esc_html__( 'Center Bottom', 'holdy' ),
            'right-bottom' => esc_html__( 'Right Bottom', 'holdy' ),
        ));
        return $alignments;
    }
    
    // Get font by id
    public function get_font_by_id($id) {
        $fonts = apply_filters('holdyFontIds',array(
            'lato' => __DIR__ . '/fonts/lato.ttf',
            'lora' => __DIR__ . '/fonts/lora.ttf',
            'montserrat' => __DIR__ . '/fonts/montserrat.ttf',
            'opensans' => __DIR__ . '/fonts/opensans.ttf',
            'oswald' => __DIR__ . '/fonts/oswald.ttf',
            'playfair' => __DIR__ . '/fonts/playfair.ttf',
            'ptsans' => __DIR__ . '/fonts/ptsans.ttf',
            'raleway' => __DIR__ . '/fonts/raleway.ttf',
            'roboto' => __DIR__ . '/fonts/roboto.ttf',
            'code' => __DIR__ . '/fonts/code.ttf',
        ));
        return $fonts[$id];
    }

    // Get supported image formats
    static function get_supported_formats() {
        $formats = apply_filters('holdySupportedFormats',array(
            'png' => esc_html__( 'PNG', 'holdy' ),
            'jpg' => esc_html__( 'JPG', 'holdy' ),
            'gif' => esc_html__( 'GIF', 'holdy' ),
            'webp' => esc_html__( 'WEBP', 'holdy' ),
            'bmp' => esc_html__( 'BMP', 'holdy' )
        ));
        return $formats;
    }

    /* Include the required file if Elementor is loaded */
    public function elementor_init(){
        if ( did_action( 'elementor/loaded' ) ) {
            include_once('elementor.php');
        }
    }
}

/**
 * Returns the main instance of the class
 */
function HoldyPlaceholder() {  
	return HoldyPlaceholder::instance();
}
// Global for backwards compatibility
$GLOBALS['HoldyPlaceholder'] = HoldyPlaceholder();