<?php
defined( 'ABSPATH' ) || exit;

class HoldySettings {
    /* The single instance of the class */
	protected static $_instance = null;

    /* Main Instance */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /* Constructor */
    public function __construct() {
        add_action( 'cmb2_admin_init', array($this, 'register_metabox') );
        add_action( 'admin_enqueue_scripts',array($this, 'colorpicker_labels'), 99 );
        add_action( 'admin_enqueue_scripts', array($this, 'admin_scripts') );
    }

    /* Admin Scripts */
    public function admin_scripts($hook){
        if ('placeholder_page_holdy_options' == $hook)  {
            wp_enqueue_style('holdy-admin', HOLDY_PLUGIN_URL . 'css/admin.css', false, HOLDY_VERSION);
            wp_enqueue_script('holdy-admin', HOLDY_PLUGIN_URL . 'js/admin.js', array( 'jquery' ), HOLDY_VERSION, true);
        } else if ('toplevel_page_holdy_placeholder' == $hook)  {
            wp_enqueue_style('spectrum', HOLDY_PLUGIN_URL . 'css/spectrum.min.css', false, '2.0.8' );
            wp_enqueue_style('holdy-admin-generator', HOLDY_PLUGIN_URL . 'css/admin-generator.css', false, HOLDY_VERSION);
            wp_enqueue_script('spectrum', HOLDY_PLUGIN_URL . 'js/spectrum.min.js', array( 'jquery' ), HOLDY_VERSION, true);
            wp_enqueue_script('holdy-admin-generator', HOLDY_PLUGIN_URL . 'js/admin-generator.js', array( 'jquery' ), HOLDY_VERSION, true);
        } else {
            return;
        }
    }

    /**
    * Hook in and register a metabox to handle a plugin options page and adds a menu item.
    */
    public function register_metabox() {
        $args = array(
            'id'           => 'holdy_options',
            'title'        => esc_html__('Holdy', 'holdy') . ' - v' . HOLDY_VERSION . ' <a href="https://elementor.thememasters.club/help/holdy/index.html" target="_blank">' . esc_html__( 'Documentation', 'holdy' ) . '</a>',
            'menu_title'   => esc_html__('Settings', 'holdy'),
            'object_types' => array( 'options-page' ),
            'option_key'   => 'holdy_options',
            'parent_slug'     => 'holdy_placeholder',
            'capability'      => 'manage_options',
            'save_button'     => esc_html__( 'Save Settings', 'holdy' )
        );

        $options = new_cmb2_box( $args );

        /* SETTINGS */
        $slug =  HoldySettings::get_option('holdy_slug', 'placeholder');
        $options->add_field( array(
            'name' => esc_html__( 'Custom URL Slug', 'holdy' ),
            'description' => esc_html__('Your current image placeholder service url is;', 'holdy') . '<br><a href="' . get_site_url() . '/' . $slug . '/" target="_blank">' . get_site_url() . '/' . $slug . '/</a>',
            'id'   => 'holdy_slug',
            'type' => 'text',
            'attributes' => array(
                'autocomplete' => 'off'
            ),
            'default' => esc_html__( 'placeholder', 'holdy' ),
        ) );

        $options->add_field( array(
            'name' => esc_html__( 'Image Size Limit', 'holdy' ),
            'description' => esc_html__('Images larger than 4000px may slow down your server.', 'holdy'),
            'id'   => 'holdy_size_limit',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
                'autocomplete' => 'off'
            ),
            'default' => 4000
        ) );

        $options->add_field( array(
            'name' => esc_html__( 'Default Values', 'holdy' ),
            'type' => 'title',
            'id'   => 'defaults-title'
        ) );

        $options->add_field( array(
            'name'    => esc_html__( 'Text', 'holdy' ),
            'description' => esc_html__('As default the image size is printed as text.', 'holdy'),
            'id'      => 'holdy_text',
            'type'    => 'textarea',
            'default' => ''
        ) );

        $options->add_field( array(
            'name'    => esc_html__( 'Image Size', 'holdy' ),
            'description' => esc_html__('Image dimensions in px. Valid format is (width)x(height). For example; 800x600', 'holdy'),
            'id'      => 'holdy_image_size',
            'type'    => 'text',
            'default' => '800x600',
        ) );

        $options->add_field( array(
            'name'    => esc_html__( 'Background', 'holdy' ),
            'description' => esc_html__('Leave blank for random color.', 'holdy'),
            'id'      => 'holdy_background_color',
            'type'    => 'colorpicker',
            'default' => '',
        ) );

        $options->add_field( array(
            'name' => esc_html__( 'Font Family', 'holdy' ),
            'id'   => 'holdy_font_family',
            'type' => 'select',
            'options' => HoldyPlaceholder::get_fonts(),
            'attributes' => array(
                'autocomplete' => 'off'
            ),
            'default' => 'lato'
        ) );

        $options->add_field( array(
            'name'    => esc_html__( 'Text Color', 'holdy' ),
            'description' => esc_html__('Leave blank for random color.', 'holdy'),
            'id'      => 'holdy_font_color',
            'type'    => 'colorpicker',
            'default' => ''
        ) );

        $options->add_field( array(
            'name' => esc_html__( 'Text Align', 'holdy' ),
            'id'   => 'holdy_align',
            'type' => 'select',
            'options' => HoldyPlaceholder::get_alignments(),
            'attributes' => array(
                'autocomplete' => 'off'
            ),
            'default' => 'center-center'
        ) );

        $options->add_field( array(
            'name'    => esc_html__( 'Stroke Color', 'holdy' ),
            'description' => esc_html__('Leave blank to disable.', 'holdy'),
            'id'      => 'holdy_stroke_color',
            'type'    => 'colorpicker',
            'default' => '',
        ) );

        $options->add_field( array(
            'name' => esc_html__( 'Stroke Size', 'holdy' ),
            'id'   => 'holdy_stroke_size',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
                'autocomplete' => 'off'
            ),
            'default' => ''
        ) );
    }
    /**
    * Colorpicker Labels
    */
    public function colorpicker_labels( $hook ) {
        global $wp_version;
        if( version_compare( $wp_version, '5.4.2' , '>=' ) ) {
            wp_localize_script(
            'wp-color-picker',
            'wpColorPickerL10n',
            array(
                'clear'            => esc_html__( 'Clear', 'holdy' ),
                'clearAriaLabel'   => esc_html__( 'Clear color', 'holdy' ),
                'defaultString'    => esc_html__( 'Default', 'holdy' ),
                'defaultAriaLabel' => esc_html__( 'Select default color', 'holdy' ),
                'pick'             => esc_html__( 'Select Color', 'holdy' ),
                'defaultLabel'     => esc_html__( 'Color value', 'holdy' )
            )
            );
        }
    }

    /**
    * Holdy get option
    */
    static function get_option( $key = '', $default = false ) {
        if ( function_exists( 'cmb2_get_option' ) ) {
            return cmb2_get_option( 'holdy_options', $key, $default );
        }
        $opts = get_option( 'holdy_options', $default );
        $val = $default;
        if ( 'all' == $key ) {
            $val = $opts;
        } elseif ( is_array( $opts ) && array_key_exists( $key, $opts ) && false !== $opts[ $key ] ) {
            $val = $opts[ $key ];
        }
        return $val;
    }

}

/**
 * Returns the main instance of the class.
 */
function HoldySettings() {  
	return HoldySettings::instance();
}
// Global for backwards compatibility.
$GLOBALS['HoldySettings'] = HoldySettings();