<?php
namespace ElementorHoldy\Widgets;

if ( ! defined( 'ABSPATH' ) ) exit;

use HoldyPlaceholder;
use HoldySettings;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;

class Holdy_Generator extends Widget_Base {

	public function get_name() {
		return 'holdy-generator';
	}

	public function get_title() {
		return esc_html__( 'Placeholder Generator', 'holdy' );
	}

	public function get_icon() {
		return 'eicon-image';
	}

	public function get_categories() {
		return [ 'holdy-widgets' ];
	}

	public function get_keywords() {
		return [ 'image', 'photo', 'visual' ];
	}

    public function get_style_depends() {
		return [ 'spectrum','holdy-generator' ];
	}
    
    public function get_script_depends() {
		return [ 'spectrum','holdy-generator' ];
	}

	protected function _register_controls() {
		$this->start_controls_section(
			'settings_section',
			[
				'label' => esc_html__( 'Layout', 'holdy' ),
                'tab' => Controls_Manager::TAB_CONTENT,
			]
		);
        
        $this->add_control(
			'layout',
			[
				'label' => esc_html__( 'Flex Direction', 'holdy' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'row',
				'label_block' => 'true',
				'options' => [
                    'row'  => esc_html__( 'Row', 'holdy' ),
					'row-reverse'  => esc_html__( 'Row Reverse', 'holdy' ),
                    'column'  => esc_html__( 'Column', 'holdy' ),
					'column-reverse'  => esc_html__( 'Column Reverse', 'holdy' )
				],
				'condition' => [
					'show_preview' => 'yes',
				],
			]
		);

        $this->add_responsive_control(
			'first_column_width',
			[
				'label' => esc_html__( 'First Column Width (%)', 'holdy' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ '%' ],
				'range' => [
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
                'default' => [
					'unit' => '%',
					'size' => 50,
				],
				'selectors' => [
					'{{WRAPPER}} .holdy-list-wrapper-row .holdy-list-left,{{WRAPPER}} .holdy-list-wrapper-row-reverse .holdy-list-left' => 'width: {{SIZE}}%;'
				],
				'condition' => [
					'show_preview' => 'yes',
				],
                'conditions'   => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name'  => 'layout',
                            'value' => 'row',
                        ],
                        [
                            'name'  => 'layout',
                            'value' => 'row-reverse',
                        ]
                    ]
                ],
			]
		);

		$this->add_responsive_control(
			'second_column_width',
			[
				'label' => esc_html__( 'Second Column Width (%)', 'holdy' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ '%' ],
				'range' => [
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
                'default' => [
					'unit' => '%',
					'size' => 50,
				],
				'selectors' => [
                    '{{WRAPPER}} .holdy-list-wrapper-row .holdy-list-right,{{WRAPPER}} .holdy-list-wrapper-row-reverse .holdy-list-right' => 'width: {{SIZE}}%;'
				],
				'condition' => [
					'show_preview' => 'yes',
				],
                'conditions'   => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name'  => 'layout',
                            'value' => 'row',
                        ],
                        [
                            'name'  => 'layout',
                            'value' => 'row-reverse',
                        ]
                    ]
                ],
			]
		);

		$this->add_control(
			'section_settings_hr',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'column_spacing',
			[
				'label' => esc_html__( 'Column Padding (px)', 'holdy' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 20,
				],
				'selectors' => [
                    '{{WRAPPER}} .holdy-list-left' => 'padding: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .holdy-list-right' => 'padding: {{SIZE}}{{UNIT}};'
				],
			]
		);

		$this->add_responsive_control(
			'gap',
			[
				'label' => esc_html__( 'Input Field Vertical Gap (px)', 'holdy' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 15,
				],
				'selectors' => [
                    '{{WRAPPER}} .holdy-form-wrapper' => 'grid-gap: {{SIZE}}{{UNIT}};'
				],
			]
		);

		$this->add_responsive_control(
			'form_column_spacing',
			[
				'label' => esc_html__( 'Input Field Horizontal Gap (px)', 'holdy' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 5,
				],
				'selectors' => [
					'{{WRAPPER}} .holdy-form-field-wrap>div' => 'padding:0 {{SIZE}}{{UNIT}} 0 {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .holdy-form-field-wrap>div:first-child' => 'padding:0 {{SIZE}}{{UNIT}} 0 0;',
					'{{WRAPPER}} .holdy-form-field-wrap>div:last-child' => 'padding:0 0 0 {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'defaults_section',
			[
				'label' => esc_html__( 'Default Values', 'holdy' ),
                'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		$allowedSize = HoldySettings::get_option('holdy_size_limit', 4000);

		$this->add_control(
			'holdy_width',
			[
				'label' => esc_html__( 'Image Width', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => $allowedSize,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

		$this->add_control(
			'holdy_height',
			[
				'label' => esc_html__( 'Image Height', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => $allowedSize,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

		$this->add_control(
			'holdy_background',
			[
				'label' => esc_html__( 'Background', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'alpha' => false,
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_image_format',
			[
				'label' => esc_html__( 'Image Format', 'holdy' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'png',
				'options' => HoldyPlaceholder::get_supported_formats(),
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_font_family',
			[
				'label' => esc_html__( 'Font Family', 'holdy' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'lato',
				'options' => HoldyPlaceholder::get_fonts(),
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_font_size',
			[
				'label' => esc_html__( 'Font Size', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 1000,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

		$this->add_control(
			'holdy_font_color',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'alpha' => false,
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_align',
			[
				'label' => esc_html__( 'Text Align', 'holdy' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'center-center',
				'options' => HoldyPlaceholder::get_alignments(),
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_stroke_color',
			[
				'label' => esc_html__( 'Stroke Color', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'alpha' => false,
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_stroke_size',
			[
				'label' => esc_html__( 'Stroke Size', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 1000,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

		$this->add_control(
			'holdy_text',
			[
				'label' => esc_html__( 'Text', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => '',
				'label_block' => true,
				'dynamic' => [
					'active' => true,
				],
				'frontend_available' => true
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'visibility_section',
			[
				'label' => esc_html__( 'Visibility', 'holdy' ),
                'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'show_preview',
			[
				'label' => esc_html__( 'Preview', 'holdy' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'holdy' ),
				'label_off' => esc_html__( 'Hide', 'holdy' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'show_size',
			[
				'label' => esc_html__( 'Size', 'holdy' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'holdy' ),
				'label_off' => esc_html__( 'Hide', 'holdy' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'show_bg_color',
			[
				'label' => esc_html__( 'Background', 'holdy' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'holdy' ),
				'label_off' => esc_html__( 'Hide', 'holdy' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'show_font_family',
			[
				'label' => esc_html__( 'Font', 'holdy' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'holdy' ),
				'label_off' => esc_html__( 'Hide', 'holdy' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'show_text_settings',
			[
				'label' => esc_html__( 'Text Format', 'holdy' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'holdy' ),
				'label_off' => esc_html__( 'Hide', 'holdy' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'show_stroke',
			[
				'label' => esc_html__( 'Stroke', 'holdy' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'holdy' ),
				'label_off' => esc_html__( 'Hide', 'holdy' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->add_control(
			'show_text',
			[
				'label' => esc_html__( 'Textarea', 'holdy' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'holdy' ),
				'label_off' => esc_html__( 'Hide', 'holdy' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'labels_section',
			[
				'label' => esc_html__( 'Labels', 'holdy' ),
                'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'holdy_preview_label',
			[
				'label' => esc_html__( 'Preview', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Preview', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_width_label',
			[
				'label' => esc_html__( 'Width', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Width', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_height_label',
			[
				'label' => esc_html__( 'Height', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Height', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_bg_label',
			[
				'label' => esc_html__( 'Background', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Background', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_format_label',
			[
				'label' => esc_html__( 'Format', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Format', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_font_family_label',
			[
				'label' => esc_html__( 'Font Family', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Font Family', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_font_size_label',
			[
				'label' => esc_html__( 'Font Size', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Font Size', 'holdy' )
			]
		);


		$this->add_control(
			'holdy_font_color_label',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Text Color', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_align_label',
			[
				'label' => esc_html__( 'Text Align', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Text Align', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_stroke_color_label',
			[
				'label' => esc_html__( 'Stroke Color', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Stroke Color', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_stroke_size_label',
			[
				'label' => esc_html__( 'Stroke Size', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Stroke Size', 'holdy' )
			]
		);

		$this->add_control(
			'holdy_text_label',
			[
				'label' => esc_html__( 'Textarea', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Text', 'holdy' )
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'btn_section',
			[
				'label' => esc_html__( 'Button', 'holdy' ),
                'tab' => Controls_Manager::TAB_CONTENT,
			]
		);
        
        $this->add_control(
			'btn_text', [
				'label' => esc_html__( 'Text', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Copy URL', 'holdy' ),
                'dynamic' => [
					'active' => true,
				],
			]
		);

		$this->add_control(
			'copied_text', [
				'label' => esc_html__( 'Copied URL!', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Copied!', 'holdy' ),
                'dynamic' => [
					'active' => true,
				],
			]
		);
        
		$this->end_controls_section();
		
		// section start
		$this->start_controls_section(
			'section_form_container_style',
			[
				'label' => esc_html__( 'Container', 'holdy' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'form_container_bg',
				'label' => esc_html__( 'Background', 'holdy' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .holdy-list-left',
			]
		);

		$this->add_control(
			'form_container_hr',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'form_container_border',
				'label' => esc_html__( 'Border', 'holdy' ),
				'selector' => '{{WRAPPER}} .holdy-list-left'
			]
		);
        
        $this->add_responsive_control(
			'form_container_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .holdy-list-left' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'form_container_shadow',
				'label' => esc_html__( 'Box Shadow', 'holdy' ),
				'selector' => '{{WRAPPER}} .holdy-list-left'
			]
		);

		$this->add_control(
			'form_container_hr_2',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);
        
        $this->add_responsive_control(
			'form_container_padding',
			[
				'label' => esc_html__( 'Padding', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .holdy-list-left' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();
		
		// section start
		$this->start_controls_section(
			'section_preview_style',
			[
				'label' => esc_html__( 'Preview', 'holdy' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'preview_border',
				'label' => esc_html__( 'Border', 'holdy' ),
				'selector' => '{{WRAPPER}} img.holdy-placeholder-img'
			]
		);
        
        $this->add_responsive_control(
			'preview_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} img.holdy-placeholder-img' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'preview_shadow',
				'label' => esc_html__( 'Box Shadow', 'holdy' ),
				'selector' => '{{WRAPPER}} img.holdy-placeholder-img'
			]
		);

		$this->end_controls_section();
		
		// section start
		$this->start_controls_section(
			'section_form_labels_style',
			[
				'label' => esc_html__( 'Form Labels', 'holdy' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);


		$this->add_control(
			'label_color',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .holdy-input-label' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'label_typography',
				'selector' => '{{WRAPPER}} .holdy-input-label',
			]
		);

		$this->add_responsive_control(
			'label_align',
			[
				'label' => esc_html__( 'Text Align', 'holdy' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'holdy' ),
						'icon' => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'holdy' ),
						'icon' => 'eicon-text-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'holdy' ),
						'icon' => 'eicon-text-align-right',
					],
				],
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .holdy-input-label' => 'text-align: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'label_hr',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'label_margin',
			[
				'label' => esc_html__( 'Margin', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .holdy-input-label' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);
        
        $this->add_responsive_control(
			'label_padding',
			[
				'label' => esc_html__( 'Padding', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .holdy-input-label' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();

		 // section start
		 $this->start_controls_section(
			'section_input_style',
			[
				'label' => esc_html__( 'Form Fields', 'holdy' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);
        
        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'input_typography',		
				'selector' => '{{WRAPPER}} input[type="text"].holdy-input-text,{{WRAPPER}} input[type="number"].holdy-input-text,{{WRAPPER}} input[type="text"].holdy-colorpicker,{{WRAPPER}} select.holdy-select,{{WRAPPER}} textarea.holdy-textarea'
			]
		);
        
        $this->add_control(
			'input_hr_1',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		); 
        
        $this->start_controls_tabs( 'form_input_style' );
        
        $this->start_controls_tab(
			'form_input_normal',
			[
				'label' => esc_html__( 'Normal', 'holdy' ),
			]
		);
        
        $this->add_control(
			'text_color',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text' => 'color: {{VALUE}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text' => 'color: {{VALUE}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker' => 'color: {{VALUE}};',
					'{{WRAPPER}} select.holdy-select' => 'color: {{VALUE}};',
					'{{WRAPPER}} textarea.holdy-textarea' => 'color: {{VALUE}};',
				]
			]
		);
        
        $this->add_control(
			'bg_color',
			[
				'label' => esc_html__( 'Background Color', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} select.holdy-select' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} select.holdy-textarea' => 'background-color: {{VALUE}};',
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'border',
				'label' => esc_html__( 'Border', 'holdy' ),
				'selector' => '{{WRAPPER}} input[type="text"].holdy-input-text,{{WRAPPER}} input[type="number"].holdy-input-text,{{WRAPPER}} input[type="text"].holdy-colorpicker,{{WRAPPER}} select.holdy-select,{{WRAPPER}} textarea.holdy-textarea'
			]
		);
        
        $this->add_responsive_control(
			'border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} select.holdy-select' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} textarea.holdy-textarea' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'border_shadow',
				'label' => esc_html__( 'Box Shadow', 'holdy' ),
				'selector' => '{{WRAPPER}} input[type="text"].holdy-input-text,{{WRAPPER}} input[type="number"].holdy-input-text,{{WRAPPER}} input[type="text"].holdy-colorpicker,{{WRAPPER}} select.holdy-select,{{WRAPPER}} textarea.holdy-textarea'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'form_input_hover',
			[
				'label' => esc_html__( 'Focus', 'holdy' ),
			]
		);
        
        $this->add_control(
			'text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text:focus' => 'color: {{VALUE}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text:focus' => 'color: {{VALUE}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker:focus' => 'color: {{VALUE}};',
					'{{WRAPPER}} textarea.holdy-textarea:focus' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'holdy' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text:focus' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text:focus' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker:focus' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} textarea.holdy-textarea:focus' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'hover_border',
				'label' => esc_html__( 'Border', 'holdy' ),
				'selector' => '{{WRAPPER}} input[type="text"].holdy-input-text:focus,{{WRAPPER}} input[type="number"].holdy-input-text:focus,{{WRAPPER}} input[type="text"].holdy-colorpicker:focus,{{WRAPPER}} textarea.holdy-textarea:focus'
			]
		);
        
        $this->add_responsive_control(
			'border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text:focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text:focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker:focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} select.holdy-select:focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} textarea.holdy-textarea:focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'holdy' ),
				'selector' => '{{WRAPPER}} input[type="text"].holdy-input-text:focus,{{WRAPPER}} input[type="number"].holdy-input-text:focus,{{WRAPPER}} input[type="text"].holdy-colorpicker:focus,{{WRAPPER}} textarea.holdy-textarea:focus'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->add_control(
			'input_hr_2',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'input_height',
			[
				'label' => esc_html__( 'Height (px)', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 10,
				'max' => 100,
				'step' => 1,
                'default' => 40,
                'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text' => 'height: {{VALUE}}px;',
					'{{WRAPPER}} input[type="number"].holdy-input-text' => 'height: {{VALUE}}px;',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker' => 'height: {{VALUE}}px;',
					'{{WRAPPER}} select.holdy-select' => 'height: {{VALUE}}px;'
				],
			]
		);
        
        $this->add_responsive_control(
			'input_padding',
			[
				'label' => esc_html__( 'Padding', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} input[type="text"].holdy-input-text' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} select.holdy-select' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} textarea.holdy-textarea' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
        
        $this->add_responsive_control(
			'input_margin',
			[
				'label' => esc_html__( 'Margin', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} input[type="text"].holdy-input-text' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="number"].holdy-input-text' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} input[type="text"].holdy-colorpicker' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} select.holdy-select' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} textarea.holdy-textarea' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
        
        $this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_btn_style',
			[
				'label' => esc_html__( 'Button', 'holdy' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);
        
        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'btn_typography',
				'selector' => '{{WRAPPER}} .holdy-btn',
			]
		);
        
        $this->add_control(
			'btn_hr_1',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);  
        
        $this->start_controls_tabs( 'tabs_button_style' );
        
        $this->start_controls_tab(
			'tab_button_normal',
			[
				'label' => esc_html__( 'Normal', 'holdy' ),
			]
		);
        
        $this->add_control(
			'btn_text_color',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::COLOR, 
                'default' => '#ffffff',
				'selectors' => [
					'{{WRAPPER}} .holdy-btn' => 'color: {{VALUE}};',
				]
			]
		);
        
        $this->add_control(
			'btn_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'holdy' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#000000',
				'selectors' => [
					'{{WRAPPER}} .holdy-btn' => 'background-color: {{VALUE}};',
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'btn_border',
				'label' => esc_html__( 'Border', 'holdy' ),
				'selector' => '{{WRAPPER}} .holdy-btn'
			]
		);
        
        $this->add_responsive_control(
			'btn_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .holdy-btn,{{WRAPPER}} .holdy-btn:before' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'btn_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'holdy' ),
				'selector' => '{{WRAPPER}} .holdy-btn'
			]
		);

		$this->add_responsive_control(
			'btn_padding',
			[
				'label' => esc_html__( 'Padding', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .holdy-btn' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'tab_button_hover',
			[
				'label' => esc_html__( 'Hover', 'holdy' ),
			]
		);
        
        $this->add_control(
			'btn_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .holdy-btn:hover' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'btn_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'holdy' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .holdy-btn:hover' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'btn_hover_border',
				'label' => esc_html__( 'Border', 'holdy' ),
				'selector' => '{{WRAPPER}} .holdy-btn:hover'
			]
		);
        
        $this->add_responsive_control(
			'btn_border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .holdy-btn:hover,{{WRAPPER}} .holdy-btn:hover:before' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'btn_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'holdy' ),
				'selector' => '{{WRAPPER}} .holdy-btn:hover'
			]
		);

		$this->add_responsive_control(
			'btn_padding_hover',
			[
				'label' => esc_html__( 'Padding', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .holdy-btn:hover' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->end_controls_section();
	}
    
    protected function render() {
		$settings = $this->get_settings_for_display();
		$widget_id = $this->get_id();
		$slug =  HoldySettings::get_option('holdy_slug', 'placeholder');
		$placeholderURL = get_site_url() . '/' . $slug . '/';
		$width = $settings['holdy_width'] ? $settings['holdy_width'] : '';
		$height = $settings['holdy_height'] ? $settings['holdy_height'] : '';
		$format = $settings['holdy_image_format'] ? $settings['holdy_image_format'] : 'png';
		$size = HoldySettings::get_option('holdy_image_size', '800x600');
		$allowedSize = HoldySettings::get_option('holdy_size_limit', 4000);

		if (!empty($width)) {
			if (!empty($height)) {
				$size = $width . 'x' . $height;
			} else {
				$size = $width;
			}
		}
		$placeholderURL = $placeholderURL . $size . '.' . $format;
		$parameters = array(
			'text' => $settings['holdy_text'] ? $settings['holdy_text'] : '',
			'font' => $settings['holdy_font_family'] ? $settings['holdy_font_family'] : '',
			'align' => $settings['holdy_align'] ? $settings['holdy_align'] : 'center-center',
			'foreground' => $settings['holdy_font_color'] ? $settings['holdy_font_color'] : '',
			'background' => $settings['holdy_background'] ? $settings['holdy_background'] : '',
			'fontsize' => $settings['holdy_font_size'] ? $settings['holdy_font_size'] : '',
			'strokecolor' => $settings['holdy_stroke_color'] ? $settings['holdy_stroke_color'] : '',
			'strokesize' => $settings['holdy_stroke_size'] ? $settings['holdy_stroke_size'] : ''
		);
		$output_parameters = '';
		foreach($parameters as $key => $parameter) {
			if (!empty($parameter) && $parameter !== 'center-center') {
			$output_parameters != "" && $output_parameters .= "&amp;";
			$output_parameters .= $key . '=' . str_replace('#', '', $parameter);
			}
		}
		if (!empty($output_parameters)) {
			$placeholderURL = $placeholderURL . '?' . $output_parameters;
		}
        ?>
        <div class="holdy-list-wrapper holdy-list-wrapper-<?php echo esc_attr($settings['layout']); ?>" data-url="<?php echo esc_url(get_site_url() . '/' . $slug . '/'); ?>" data-size="<?php echo esc_attr($size); ?>">
			<?php if ( 'yes' === $settings['show_preview'] ) { ?>
			<div class="holdy-list-left">
				<?php if ($settings['holdy_preview_label']) { ?>
				<label class="holdy-input-label"><?php echo esc_html($settings['holdy_preview_label']); ?></label>
				<?php } ?>
				<img class="holdy-placeholder-img" src="<?php echo esc_url($placeholderURL); ?>" />
			</div>
			<?php } ?>
            <div class="holdy-list-right <?php if ( 'yes' !== $settings['show_preview'] ) { echo 'full-row'; } ?>">
				<div id="holdy-list-form-<?php echo esc_attr($widget_id); ?>" class="holdy-form-wrapper">
				<div class="holdy-form-field-wrap <?php if ( 'yes' !== $settings['show_size'] ) { echo 'holdy-hidden'; } ?>">
					<div class="holdy-form-field">
						<?php if ($settings['holdy_width_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_width_label']); ?></label>
						<?php } ?>
						<input class="holdy-input-text holdy-numeric" type="number" name="holdy_width" value="<?php echo esc_attr($width); ?>" autocomplete="off" max="<?php echo esc_attr($allowedSize); ?>" >
					</div>
					<div class="holdy-form-field">
						<?php if ($settings['holdy_height_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_height_label']); ?></label>
						<?php } ?>
						<input class="holdy-input-text holdy-numeric" type="number" name="holdy_height" value="<?php echo esc_attr($height); ?>" autocomplete="off" max="<?php echo esc_attr($allowedSize); ?>" >
					</div>
				</div>
				<div class="holdy-form-field-wrap <?php if ( 'yes' !== $settings['show_bg_color'] ) { echo 'holdy-hidden'; } ?>">
					<div class="holdy-form-field">
						<?php if ($settings['holdy_bg_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_bg_label']); ?></label>
						<?php } ?>
						<input class="holdy-colorpicker" type="text" name="holdy_background" value="<?php echo esc_attr($parameters['background']); ?>" autocomplete="off">
					</div>
					<div class="holdy-form-field">
						<?php if ($settings['holdy_format_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_format_label']); ?></label>
						<?php } ?>
						<select class="holdy-select" name="holdy_format" autocomplete="off">
						<?php 
						$formats = HoldyPlaceholder::get_supported_formats(); 
						foreach($formats as $key => $item) {
							$selected = '';
							if ($key == $format) {
								$selected = 'selected';
							}
							echo '<option value="' . esc_attr($key) . '" ' . esc_attr($selected) . '>' . esc_html($item) . '</option>';
						}
						?>
						</select>
					</div>
				</div>
				<div class="holdy-form-field-wrap <?php if ( 'yes' !== $settings['show_font_family'] ) { echo 'holdy-hidden'; } ?>">
					<div class="holdy-form-field">
						<?php if ($settings['holdy_font_family_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_font_family_label']); ?></label>
						<?php } ?>
						<select class="holdy-select" name="holdy_font_family" autocomplete="off">
						<option value=""><?php echo esc_html__( 'Default', 'holdy' ); ?></option>
						<?php 
						$font_family_array = HoldyPlaceholder::get_fonts(); 
						foreach($font_family_array as $key => $item) {
							$selected = '';
							if ($key == $parameters['font']) {
								$selected = 'selected';
							}
							echo '<option value="' . esc_attr($key) . '" ' . esc_attr($selected) . '>' . esc_html($item) . '</option>';
						}
						?>
						</select>
					</div>
					<div class="holdy-form-field">
						<?php if ($settings['holdy_font_size_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_font_size_label']); ?></label>
						<?php } ?>
						<input class="holdy-input-text holdy-numeric" type="number" name="holdy_font_size" value="<?php echo esc_attr($parameters['fontsize']); ?>" autocomplete="off" max="1000">
					</div>
				</div>
				<div class="holdy-form-field-wrap <?php if ( 'yes' !== $settings['show_text_settings'] ) { echo 'holdy-hidden'; } ?>">
					<div class="holdy-form-field">
						<?php if ($settings['holdy_font_color_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_font_color_label']); ?></label>
						<?php } ?>
						<input class="holdy-colorpicker" type="text" name="holdy_font_color" value="<?php echo esc_attr($parameters['foreground']); ?>" autocomplete="off">
					</div>
					<div class="holdy-form-field">
						<?php if ($settings['holdy_align_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_align_label']); ?></label>
						<?php } ?>
						<select class="holdy-select" name="holdy_align" autocomplete="off">
						<?php 
						$alignment_array = HoldyPlaceholder::get_alignments(); 
						foreach($alignment_array as $key => $item) {
							$selected = '';
							if ($key == $parameters['align']) {
								$selected = 'selected';
							}
							echo '<option value="' . esc_attr($key) . '" ' . esc_attr($selected) . '>' . esc_html($item) . '</option>';
						}
						?>
						</select>
					</div>
				</div>
				<div class="holdy-form-field-wrap <?php if ( 'yes' !== $settings['show_text_settings'] ) { echo 'holdy-hidden'; } ?>">
					<div class="holdy-form-field">
						<?php if ($settings['holdy_stroke_color_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_stroke_color_label']); ?></label>
						<?php } ?>
						<input class="holdy-colorpicker" type="text" name="holdy_stroke_color" value="<?php echo esc_attr($parameters['strokecolor']); ?>" autocomplete="off">
					</div>
					<div class="holdy-form-field">
						<?php if ($settings['holdy_stroke_size_label']) { ?>
						<label class="holdy-input-label"><?php echo esc_html($settings['holdy_stroke_size_label']); ?></label>
						<?php } ?>
						<input class="holdy-input-text holdy-numeric" type="number" name="holdy_stroke_size" value="<?php echo esc_attr($parameters['strokesize']); ?>" autocomplete="off" max="<?php echo esc_attr($allowedSize); ?>" >
					</div>
				</div>
				<div class="holdy-form-field <?php if ( 'yes' !== $settings['show_text'] ) { echo 'holdy-hidden'; } ?>">
					<?php if ($settings['holdy_text_label']) { ?>
					<label class="holdy-input-label"><?php echo esc_html($settings['holdy_text_label']); ?></label>
					<?php } ?>
					<textarea class="holdy-textarea" name="holdy_text" autocomplete="off"><?php echo esc_html($parameters['text']); ?></textarea>
				</div>
				<div class="holdy-form-field-wrap copy-field">
					<div class="holdy-form-field">
						<input class="holdy-input-text" type="text" name="holdy_placeholder_url" value="<?php echo esc_url($placeholderURL); ?>" autocomplete="off" readonly>
					</div>
					<div class="holdy-form-field">
						<button type="button" class="holdy-btn" name="holdy_copy_url" data-text="<?php echo esc_attr($settings['btn_text']); ?>" data-copied="<?php echo esc_attr($settings['copied_text']); ?>"><?php echo esc_html($settings['btn_text']); ?></button>
					</div>
				</div>
				</div>
			</div>
		</div>
<?php
    }
}
?>