<?php

namespace ElementorHoldy\Widgets;

if ( ! defined( 'ABSPATH' ) ) exit;

use HoldyPlaceholder;
use HoldySettings;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;

/**
 * Elementor image widget.
 *
 * Elementor widget that displays an image into the page.
 *
 * @since 1.0.0
 */
class Holdy_Image extends Widget_Base {

	public function get_name() {
		return 'holdy-image';
	}

	public function get_title() {
		return esc_html__( 'Placeholder Image', 'holdy' );
	}

	public function get_icon() {
		return 'eicon-image';
	}

    public function get_style_depends() {
		return [ 'holdy-image' ];
	}

	public function get_categories() {
		return [ 'holdy-widgets' ];
	}

	public function get_keywords() {
		return [ 'image', 'photo', 'visual' ];
	}

	protected function register_controls() {
		$this->start_controls_section(
			'section_image',
			[
				'label' => esc_html__( 'Image', 'holdy' ),
			]
		);

		$allowedSize = HoldySettings::get_option('holdy_size_limit', 4000);

		$this->add_control(
			'holdy_width',
			[
				'label' => esc_html__( 'Image Width', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => $allowedSize,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

		$this->add_control(
			'holdy_height',
			[
				'label' => esc_html__( 'Image Height', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => $allowedSize,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

		$this->add_control(
			'holdy_background',
			[
				'label' => esc_html__( 'Background', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'alpha' => false,
				'frontend_available' => true
			]
        );

        $this->add_control(
			'holdy_image_format',
			[
				'label' => esc_html__( 'Image Format', 'holdy' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'png',
				'options' => HoldyPlaceholder::get_supported_formats(),
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_font_family',
			[
				'label' => esc_html__( 'Font Family', 'holdy' ),
				'type' => Controls_Manager::SELECT,
				'default' => '',
				'options' => HoldyPlaceholder::get_fonts(),
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_font_size',
			[
				'label' => esc_html__( 'Font Size', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 1000,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

		$this->add_control(
			'holdy_font_color',
			[
				'label' => esc_html__( 'Text Color', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'alpha' => false,
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_align',
			[
				'label' => esc_html__( 'Text Align', 'holdy' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'center-center',
				'options' => HoldyPlaceholder::get_alignments(),
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_stroke_color',
			[
				'label' => esc_html__( 'Stroke Color', 'holdy' ),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'alpha' => false,
				'frontend_available' => true
			]
        );

		$this->add_control(
			'holdy_stroke_size',
			[
				'label' => esc_html__( 'Stroke Size', 'holdy' ),
				'type' => Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 1000,
				'step' => 1,
				'default' => '',
				'frontend_available' => true
			]
		);

        $this->add_responsive_control(
			'holdy_img_align',
			[
				'label' => esc_html__( 'Image Alignment', 'holdy' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'flex-start' => [
						'title' => esc_html__( 'Left', 'holdy' ),
						'icon' => 'eicon-h-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'holdy' ),
						'icon' => 'eicon-h-align-center',
					],
					'flex-end' => [
						'title' => esc_html__( 'Right', 'holdy' ),
						'icon' => 'eicon-h-align-right',
					],
				],
				'selectors' => [
					'{{WRAPPER}} .holdy-img-wrap' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'holdy_text',
			[
				'label' => esc_html__( 'Text', 'holdy' ),
				'type' => Controls_Manager::TEXT,
				'default' => '',
				'label_block' => true,
                'dynamic' => [
					'active' => true,
				],
				'frontend_available' => true
			]
		);

        $this->end_controls_section();
		
		// section start
		$this->start_controls_section(
			'section_image_style',
			[
				'label' => esc_html__( 'Image', 'holdy' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'image_border',
				'selector' => '{{WRAPPER}} .holdy-img-wrap img',
				'separator' => 'before',
			]
		);

		$this->add_responsive_control(
			'image_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'holdy' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors' => [
					'{{WRAPPER}} .holdy-img-wrap img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'image_box_shadow',
				'exclude' => [
					'box_shadow_position',
				],
				'selector' => '{{WRAPPER}} .holdy-img-wrap img',
			]
		);

        $this->end_controls_tab();
        $this->end_controls_tabs();

		$this->end_controls_section();
	}

	protected function render() {
		$settings = $this->get_settings_for_display();

        $slug =  HoldySettings::get_option('holdy_slug', 'placeholder');
		$placeholderURL = get_site_url() . '/' . $slug . '/';
		$width = $settings['holdy_width'] ? $settings['holdy_width'] : '';
		$height = $settings['holdy_height'] ? $settings['holdy_height'] : '';
		$format = $settings['holdy_image_format'] ? $settings['holdy_image_format'] : 'png';
		$size = HoldySettings::get_option('holdy_image_size', '800x600');
		$allowedSize = HoldySettings::get_option('holdy_size_limit', 4000);

		if (!empty($width)) {
			if (!empty($height)) {
				$size = $width . 'x' . $height;
			} else {
				$size = $width;
			}
		}
		$placeholderURL = $placeholderURL . $size . '.' . $format;
		$parameters = array(
			'text' => $settings['holdy_text'] ? $settings['holdy_text'] : '',
			'font' => $settings['holdy_font_family'] ? $settings['holdy_font_family'] : '',
			'align' => $settings['holdy_align'] ? $settings['holdy_align'] : 'center-center',
			'foreground' => $settings['holdy_font_color'] ? $settings['holdy_font_color'] : '',
			'background' => $settings['holdy_background'] ? $settings['holdy_background'] : '',
			'fontsize' => $settings['holdy_font_size'] ? $settings['holdy_font_size'] : '',
			'strokecolor' => $settings['holdy_stroke_color'] ? $settings['holdy_stroke_color'] : '',
			'strokesize' => $settings['holdy_stroke_size'] ? $settings['holdy_stroke_size'] : ''
		);
		$output_parameters = '';
		foreach($parameters as $key => $parameter) {
			if (!empty($parameter)) {
			$output_parameters != "" && $output_parameters .= "&amp;";
			$output_parameters .= $key . '=' . str_replace('#', '', $parameter);
			}
		}
		if (!empty($output_parameters)) {
			$placeholderURL = $placeholderURL . '?' . $output_parameters;
		}

        echo '<div class="holdy-img-wrap"><img src="' . esc_url($placeholderURL) . '" /></div>';
	}
}
